package com.qy.worksheetsystem.service.impl;


import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.kingdee.shr.api.OSFWSClient;
import com.kingdee.shr.api.Response;
import com.kingdee.shr.api.SHRClient;
import com.qy.worksheetsystem.model.entity.JobBooking;
import com.qy.worksheetsystem.model.entity.JobBookingEntry;
import com.qy.worksheetsystem.model.mapper.PersonWorkMapper;

import com.qy.worksheetsystem.model.mapper.ProjectMapper;
import com.qy.worksheetsystem.service.PersonService;
import com.qy.worksheetsystem.service.PersonWorkService;
import com.qy.worksheetsystem.util.SnowflakeUtils;
import com.qy.worksheetsystem.vo.WorkHoursReport;
import com.qy.worksheetsystem.vo.WorkHoursReportEntry;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.HttpException;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.io.IOException;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @BelongsProject: workSheetSystem
 * @BelongsPackage: com.qy.worksheetsystem.service.impl
 * @Author: lhbj
 * @CreateTime: 2024-12-03  15:18
 * @Description: TODO
 * @Version: 1.0
 */
@Slf4j
@Service
public class PersonWorkServiceImpl implements PersonWorkService {
    private static final BigDecimal h8 = new BigDecimal(8);

    @Resource
    private PersonWorkMapper personWorkMapper;
    @Resource
    private PersonService personService;
    @Resource
    private ProjectMapper projectMapper;

    @Value("${HR.ais.url}")
    private String url;
    @Value("${HR.ais.serviceName}")
    private String serviceName;
    @Value("${HR.ais.codingRuleService}")
    private String codingRuleService;

    @Override
    @Transactional
    public Map<String, Object> insertWork(Map<String, Object> param) {
        Map<String, Object> result = null;
        SHRClient shrClient = null;
        try {
            Map<String, String> person = (Map<String, String>) param.get("person");
            shrClient = new SHRClient(url, person.get("userNumber"));
            result = this.insertWork(param, shrClient);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        } finally {
            try {
                if (null != shrClient)
                    shrClient.logout(url, shrClient.getGlobalToken(), shrClient.getGlobalHttpClient());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
            }
        }
        return result;
    }

    @Override
    @Transactional
    public Map<String, Object> insertWork(Map<String, Object> param, SHRClient shrClient) {
        List<Map<String, Object>> billList = Lists.newArrayList();
        List<Map<String, Object>> workList = Lists.newArrayList();
        int begin = (int) param.get("begin");
        int end = (int) param.get("end");
        String yearMonth = (String) param.get("yearMonth");
        WorkHoursReport jobWork = (WorkHoursReport) param.get("jobWork");
        Map<String, String> person = (Map<String, String>) param.get("person");
        Date beginDate = (Date) param.get("beginDate");
        Date endDate = (Date) param.get("endDate");
        Map<String, Map<String, Object>> availableMap = (Map<String, Map<String, Object>>) param.get("availableMap");
        Map<String, Object> reMap = Maps.newHashMap();
        try {
            SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
            Integer Hours = 0;
            for (WorkHoursReportEntry jobWorkEntry : jobWork.getEntries()) {
                String workingHours = jobWorkEntry.getWorkingHours();
                BigDecimal Hour = new BigDecimal(workingHours);
                Hours += Hour.intValue();
                String projectId = jobWorkEntry.getProjectId();
                Map<String, String> pros = personService.getProjectManagementInfoBYId(projectId, person.get("id"));
                if (null==pros&&pros.isEmpty()) {
                    throw new Exception("没有找到项目,请检查");
                }
                String number = SnowflakeUtils.getInstance().getSnowflake().nextIdStr();
                Map<String, Object> map = Maps.newHashMap();
                String str = "";

                Response response = shrClient.executeServiceByLongConnection(url, codingRuleService, map);
                str = (String) response.getData();
                if (StrUtil.isNotBlank(str)) {
                    if (JSONUtil.isTypeJSON(str)) {
                        JSONObject j = JSONUtil.parseObj(str);
                        String code = j.getStr("code");
                        if ("200".equals(code)) {
                            number = j.getStr("number");
                        }
                    }
                }

                Map<String, Object> billMap = new HashMap<>();
                String billID = personWorkMapper.selectBillKey(String.valueOf(number));
                billMap.put("id", billID);
                billMap.put("affiliatedProject", projectId);
                billMap.put("fillingStartDate", jobWork.getBeginTime());
                billMap.put("fillingEndDate", jobWork.getEndTime());
                billMap.put("adminOrg", person.get("adminOrgId"));
                billMap.put("hrOrgUnit", person.get("hrOrgUnitID"));
                billMap.put("billState", "0");
                billMap.put("personId", person.get("id"));
                billMap.put("ptypeId", pros.get("ptypeId"));
                billMap.put("projectRoleId", pros.get("projectRoleId"));
                billMap.put("number", number);
                billMap.put("CU", person.get("controlUnitID"));
                Date createTime = new Date();
                billMap.put("creator", person.get("userID"));
                billMap.put("lastUpdateUser", person.get("userID"));
                billMap.put("jobContent", jobWorkEntry.getJobContent());
                BigDecimal WorkingHoursSum = BigDecimal.ZERO;
                for (int i = begin; i <= end; i++) {
                    Map<String, Object> objectMap = Maps.newHashMap();
                    String iday = "01";
                    if (i < 10) {
                        iday = "0" + i;
                    } else {
                        iday = i + "";
                    }
                    String reportDate = yearMonth + iday;
                    Map<String, Object> avMap = availableMap.get(reportDate);
                    BigDecimal availableWorkHous = (BigDecimal) avMap.get("availableWorkHours");
                    BigDecimal workHours = (BigDecimal) avMap.get("workHours");
                    objectMap.put("parent", billID);
                    objectMap.put("employee", person.get("id"));
                    objectMap.put("clockTime", reportDate);
                    objectMap.put("clockLocation", "移动端");
                    objectMap.put("data", reportDate);
                    objectMap.put("WorkingHours", Hour);
                    objectMap.put("notes", "");
                    objectMap.put("seq", i);
                    workList.add(objectMap);
                    //WorkingHoursSum=WorkingHoursSum.add(Hour);
                }
                billMap.put("WorkingHours", Hour);
                billList.add(billMap);
            }

            int bInt = personWorkMapper.insertBill(billList);
            int wInt = personWorkMapper.insertWork(workList);
            reMap.put("billList", billList);
            reMap.put("workList", workList);
            reMap.put("code", (billList.size() + workList.size()) == (bInt + wInt));
            return reMap;
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            for (Map<String, Object> bill : billList) {
                this.deleteWork(person.get("personNumber"), (String) bill.get("id"));
            }
            reMap.put("errMsg", e.getMessage());
            reMap.put("code", false);
        }
        return reMap;
    }

    @Override
    public List<Map<String, Object>> getAvailableDateInfoById(String number, String id, String startDate, String endDate) {
        Map<String, String> person = personService.getPersonByJobNo(number);
        List<Map<String, Object>> list = personWorkMapper.getAvailableDateInfoById(person.get("id"), id, startDate, endDate);
        for (Map<String, Object> map : list) {

            BigDecimal workHours = (BigDecimal) map.get("workHours");
            if (null != workHours) {
                map.put("availableWorkHours", h8.subtract(workHours));
            } else {
                map.put("workHours", BigDecimal.ZERO);
                map.put("availableWorkHours", h8);
            }
        }
        return list;
    }

    @Override
    public List<Map<String, Object>> getAvailableDateInfo(String number, String startDate, String endDate) {
        Map<String, String> person = personService.getPersonByJobNo(number);
        List<Map<String, Object>> list = personWorkMapper.getAvailableDateInfo(person.get("id"), startDate, endDate);
        for (Map<String, Object> map : list) {

            BigDecimal workHours = (BigDecimal) map.get("workHours");
            if (null != workHours) {
                map.put("availableWorkHours", h8.subtract(workHours));
            } else {
                map.put("workHours", BigDecimal.ZERO);
                map.put("availableWorkHours", h8);
            }
        }
        return list;
    }

    @Override
    public List<Map<String, Object>> getPersonWorkList(String number, String beginTime, String endTime, Boolean state) {
        List<Map<String, Object>> list = personWorkMapper.getPersonWorkList(number, beginTime, endTime, state);
        return list;
    }

    @Override
    public Map<String, Object> getPersonWorkListV3(String number, String beginTime, String endTime, Boolean state) {
        List<JobBooking> list = personWorkMapper.getPersonWorkListV2(number, beginTime, endTime, state);
        BigDecimal unapprovedHours = BigDecimal.ZERO;
        BigDecimal approveHours = BigDecimal.ZERO;
        BigDecimal workingHours = BigDecimal.ZERO;
        BigDecimal work8 = new BigDecimal(8);
        Map<String, Object> map = new HashMap<>();
        for (JobBooking job : list) {
            BigDecimal hours = BigDecimal.ZERO;
            BigDecimal days = BigDecimal.ZERO;
            String billState = String.valueOf(job.get("state"));
            for (JobBookingEntry jobEntry : (List<JobBookingEntry>) job.get("entries")) {
                BigDecimal work = new BigDecimal(jobEntry.getWorkingHours());
                workingHours = workingHours.add(work);
                hours = work;
                days = days.add(work);
                if ("3".equals(billState)) {
                    approveHours = approveHours.add(work);
                } else {
                    unapprovedHours = unapprovedHours.add(work);
                }
            }
            job.put("WorkingHours", hours.toPlainString());
            job.put("days", days.divide(work8, 4));
        }
        map.put("datalist", list);
        map.put("approveDays", approveHours.divide(work8, 4));
        map.put("unapprovedDays", unapprovedHours.divide(work8, 4));
        map.put("workingDays", workingHours.divide(work8, 4));
        return map;
    }

    @Override
    public List<JobBooking> getPersonWorkListV2(String number, String beginTime, String endTime, Boolean state) {
        List<JobBooking> list = personWorkMapper.getPersonWorkListV2(number, beginTime, endTime, state);
        BigDecimal workingHours = BigDecimal.ZERO;
        for (JobBooking job : list) {

            BigDecimal hours = BigDecimal.ZERO;
            for (JobBookingEntry jobEntry : (List<JobBookingEntry>) job.get("entries")) {
                BigDecimal work = new BigDecimal(jobEntry.getWorkingHours());
                workingHours = workingHours.add(work);
                hours = work;
            }
            job.put("WorkingHours", hours.toPlainString());
        }
        return list;
    }

    @Override
    public List<Map<String, Object>> getPersonWorkSumInfo(String number, String beginTime, String endTime, Boolean state) {
        List<Map<String, Object>> list = personWorkMapper.getPersonWorkSumInfo(number, beginTime, endTime, state);
        return list;
    }

    @Override
    public JobBooking getPersonWorkInfoById(String number, String id) {
        JobBooking e = personWorkMapper.getPersonWorkInfoById(number, id);
        BigDecimal workingHours = BigDecimal.ZERO;

        for (JobBookingEntry jobEntry : (List<JobBookingEntry>) e.get("entries")) {

            workingHours =new BigDecimal(jobEntry.getWorkingHours());

        }
        e.put("WorkingHours", workingHours.toPlainString());
        return e;
    }

    @Override
    @Transactional
    public int deleteWork(String number, String id) {
        int x = personWorkMapper.deleteWorkEntry(id);
        int i = personWorkMapper.deleteWork(id);
        return i;
    }

    @Override
    @Transactional
    public int submitWork(String number, String id, SHRClient shrClient) {
        Map<String, Object> map = Maps.newHashMap();
        map.put("billId", id);//  单据id
        map.put("status", "1");//  单据id
        try {
            Response response = shrClient.executeServiceByLongConnection(url, serviceName, map);
            String str = (String) response.getData();
            if (StrUtil.isNotBlank(str)) {
                if (JSONUtil.isTypeJSON(str)) {
                    JSONObject j = JSONUtil.parseObj(str);
                    String code = j.getStr("code");
                    if ("200".equals(code)) {
                        return 1;
                    }
                }
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
        return 0;
    }

    @Override
    @Transactional
    public int submitWork(String number, String id) {
        SHRClient shrClient = null;
        int result =0;
        try {
            shrClient = new SHRClient(url, number);
            result= this.submitWork(number, id, shrClient);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        } finally {
            try {
                if (null != shrClient)
                    shrClient.logout(url, shrClient.getGlobalToken(), shrClient.getGlobalHttpClient());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
            }
        }
        return result;
    }

    @Override
    @Transactional
    public int revocaWork(String number, String id, SHRClient shrClient) {

        Map<String, Object> map = Maps.newHashMap();
        map.put("billId", id);//  单据id
        map.put("status", "2");//  单据id
        try {
            Response response = shrClient.executeServiceByLongConnection(url, serviceName, map);
            String str = (String) response.getData();
            if (StrUtil.isNotBlank(str)) {
                if (JSONUtil.isTypeJSON(str)) {
                    JSONObject j = JSONUtil.parseObj(str);
                    String code = j.getStr("code");
                    if ("200".equals(code)) {
                        return 1;
                    }
                } else {
                    return 0;
                }
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
        return 0;
    }

    @Override
    @Transactional
    public int revocaWork(String number, String id) {
        SHRClient shrClient = null;
        int result=0;
        try {
            shrClient = new SHRClient(url, number);
            result= this.revocaWork(number, id, shrClient);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        } finally {
            try {
                if (null != shrClient)
                    shrClient.logout(url, shrClient.getGlobalToken(), shrClient.getGlobalHttpClient());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
            }
        }
        return result;
    }

    @Override
    @Transactional
    public Map<String, Object> deleteWorkToMap(String number, String id) {
        int x = personWorkMapper.deleteWorkEntry(id);
        int i = personWorkMapper.deleteWork(id);
        Map<String, Object> map = Maps.newHashMap();
        map.put("code", (i > 0 ? 1 : 0));
        map.put("work", i);
        map.put("workEntry", x);
        return map;
    }
    @Override
    @Transactional
    public Map<String, Object> submitWorkToMap(String number, String id, SHRClient shrClient) {
        Map<String, Object> map = Maps.newHashMap();
        map.put("billId", id);//  单据id
        map.put("status", "1");//  单据id
        try {
            Response response = shrClient.executeServiceByLongConnection(url, serviceName, map);
            String str = (String) response.getData();
            map.put("code", 0);
            if (StrUtil.isNotBlank(str)) {
                if (JSONUtil.isTypeJSON(str)) {
                    JSONObject j = JSONUtil.parseObj(str);
                    String code = j.getStr("code");
                    if ("200".equals(code)) {
                        map.put("code", 1);
                    }
                } else {
                    map.put("msg", str);
                }
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
        return map;
    }
    @Override
    @Transactional
    public Map<String, Object> submitWorkToMap(String number, String id) {
        Map<String, Object> result=null;
        SHRClient shrClient = null;
        try {
            shrClient = new SHRClient(url, number);
            result= this.submitWorkToMap(number, id, shrClient);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        } finally {
            try {
                if (null != shrClient)
                    shrClient.logout(url, shrClient.getGlobalToken(), shrClient.getGlobalHttpClient());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
            }
        }
        return result;
    }
    @Override
    @Transactional
    public Map<String, Object> revocaWorkToMap(String number, String id, SHRClient shrClient) {
        Map<String, Object> map = Maps.newHashMap();
        map.put("billId", id);//  单据id
        map.put("status", "2");//  单据id
        try {
            Response response = shrClient.executeServiceByLongConnection(url, serviceName, map);
            String str = (String) response.getData();
            map.put("code", 0);
            if (StrUtil.isNotBlank(str)) {
                if (JSONUtil.isTypeJSON(str)) {
                    JSONObject j = JSONUtil.parseObj(str);
                    String code = j.getStr("code");
                    if ("200".equals(code)) {
                        map.put("code", 1);
                    }
                } else {
                    map.put("msg", str);
                }
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
        return map;
    }
    @Override
    @Transactional
    public Map<String, Object> revocaWorkToMap(String number, String id) {
        Map<String, Object> result=null;
        SHRClient shrClient = null;
        try {
            shrClient = new SHRClient(url, number);
            result= this.revocaWorkToMap(number, id, shrClient);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        } finally {
            try {
                if (null != shrClient)
                    shrClient.logout(url, shrClient.getGlobalToken(), shrClient.getGlobalHttpClient());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
            }
        }
        return result;
    }
}
