package com.kingdee.eas.hr.perf.utils;

import com.kingdee.bos.BOSException;
import com.kingdee.bos.Context;
import com.kingdee.bos.dao.IObjectPK;
import com.kingdee.bos.dao.ObjectNotFoundException;
import com.kingdee.bos.dao.ormapping.ObjectUuidPK;
import com.kingdee.bos.metadata.entity.*;
import com.kingdee.bos.metadata.query.util.CompareType;
import com.kingdee.eas.common.EASBizException;
import com.kingdee.eas.framework.CoreBaseCollection;
import com.kingdee.shr.base.syssetting.api.bean.BatchMessageTipsBody;
import com.kingdee.shr.base.syssetting.exception.SHRWebException;
import com.kingdee.shr.base.syssetting.exception.ShrWebBizException;
import com.kingdee.shr.base.syssetting.util.LocaleUtils;
import com.kingdee.shr.perfweb.app.base.evalfile.PerfFileCollection;
import com.kingdee.shr.perfweb.app.base.evalfile.PerfFileFactory;
import com.kingdee.shr.perfweb.app.base.evalfile.PerfFileInfo;
import com.kingdee.shr.perfweb.app.base.evalplan.*;
import com.kingdee.shr.perfweb.app.base.evalplan.RefStatusEnum;
import com.kingdee.shr.perfweb.app.base.evalrule.*;
import com.kingdee.shr.perfweb.app.base.perfmancescore.PerfScoreInfo;
import com.kingdee.shr.perfweb.app.common.YesOrNoEnum;
import com.kingdee.shr.perfweb.app.exception.SHRPerfWebBizException;
import com.kingdee.shr.perfweb.app.resource.SHRPerfWebResource;
import com.kingdee.shr.perfweb.util.*;
import com.kingdee.shr.perfweb.utils.PerfPlanUtils;
import com.kingdee.shr.perfweb.utils.RuleHistoryUtils;
import com.kingdee.util.StringUtils;
import org.apache.log4j.Logger;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.util.*;

/**
 * @author qingwu
 * @date 2024/12/25
 * @apiNote
 */
public class PerfEvaObjectBeanUtils {
    private static Logger logger =
            Logger.getLogger(PerfEvaObjectBeanUtils.class);

    /**
     * @param ctx
     * @param perfFiles ԱԱ
     * @param periodId  id
     * @param hrOrgUnit Ĭϣ00000000-0000-0000-0000-000000000000CCE7AED4
     * @param perfPlan  ˼ƻID
     * @throws SHRWebException
     */
    public void addFromPerfFile(Context ctx, String perfFiles, String periodId, String hrOrgUnit, PerfPlanInfo perfPlan) throws SHRWebException {
        logger.error("addFromPerfFile----" + periodId);
        String billId = perfFiles;
        //String billId = "nJsYSsgcTvqTNTFmY+YCRFulBK0=";
        checkPeriod(ctx, periodId);
        String[] ids = null;
        ids = billId.split(",");
        Set<String> set = new HashSet();
        set.addAll(Arrays.asList(ids));
        EntityViewInfo viewInfo = new EntityViewInfo();
        FilterInfo filter = new FilterInfo();
        SelectorItemCollection sic = viewInfo.getSelector();
        sic.add(new SelectorItemInfo("*"));
        sic.add(new SelectorItemInfo("position.id"));
        sic.add(new SelectorItemInfo("adminOrgUnit.id"));
        sic.add(new SelectorItemInfo("hrOrgUnit.id"));
        sic.add(new SelectorItemInfo("person.number"));
        sic.add(new SelectorItemInfo("person.name"));
        sic.add(new SelectorItemInfo("empPosOrgRelation.*"));
        filter.getFilterItems().add(new FilterItemInfo("id", set, CompareType.INCLUDE));
        viewInfo.setFilter(filter);
        PerfFileCollection coll = null;
        PerfGoalsGroupCollection groupColl = null;
        String groupId = "";
        PWorkFlowDefInfo defaultWorkFlow = null;

        try {
            coll = PerfFileFactory.getLocalInstance(ctx).getPerfFileCollection(viewInfo);
            groupColl = PerfGoalsGroupFactory.getLocalInstance(ctx).getPerfGoalsGroupCollection("where period = '" + periodId + "' and useDefault = 1");
            if (groupColl != null && groupColl.get(0) != null) {
                groupId = groupColl.get(0).getId().toString();
            }

            PerfPeriodInfo perfPeriodInfo = PerfPeriodFactory.getLocalInstance(ctx).getPerfPeriodInfo("where id = '" + periodId + "'");
            PerformanceRuleInfo perfRule = perfPeriodInfo.getPerfRule();
            PWorkFlowDefCollection pWFDefColl = RuleHistoryUtils.getDefaultWorkFlow(ctx, perfRule.getId().toString());
            if (pWFDefColl != null && pWFDefColl.size() > 0) {
                defaultWorkFlow = pWFDefColl.get(0);
            } else if (perfPeriodInfo.getIsHistoryUpdate().getValue() == 1) {
                defaultWorkFlow = perfPlan.getDefaultWorkFlow();
            }
        } catch (BOSException var20) {
            var20.printStackTrace();
            logger.error(var20.toString());
        } catch (EASBizException var21) {
            var21.printStackTrace();
            logger.error(var21.toString());
        }

        if (coll != null && defaultWorkFlow != null) {
            CoreBaseCollection croeBase = new CoreBaseCollection();

            for (int i = 0; i < coll.size(); ++i) {
                PerfEvaObjectInfo info = new PerfEvaObjectInfo();
                PerfFileInfo fileInfo = coll.get(i);
                info.put("number", fileInfo.getPerson().getNumber());
                info.setName(fileInfo.getPerson().getName());
                info.setName(fileInfo.getPerson().getName(LocaleUtils.locale_l1), LocaleUtils.locale_l1);
                info.setName(fileInfo.getPerson().getName(LocaleUtils.locale_l2), LocaleUtils.locale_l2);
                info.setName(fileInfo.getPerson().getName(LocaleUtils.locale_l3), LocaleUtils.locale_l3);
                info.put("period", periodId);
                info.setCanAdjustEvaluer(TargetEvaluModeEnum.getEnum(perfPlan.getCanAdjustEvaluer().getValue()));
                info.setHrUnitOrg(fileInfo.getHrOrgUnit());
                info.setPerfFile(fileInfo);
                info.setAdminOrgUnit(fileInfo.getAdminOrgUnit());
                info.put("person", fileInfo.getPerson());
                info.put("joinPeriodDate", new Date());
                info.put("joinPlanDate", new Date());
                info.put("changeDealType", 100);
                info.put("workFlow", defaultWorkFlow.getId().toString());
                info.put("unitGoalsStatus", "0");
                info.put("belongGroup", groupId);
                info.setScoreTaskStatus(ScoreTaskStatusEnum.UNSEND);
                info.setTargetMakeSure(perfPlan.isTargetMakeSure() ? 1 : 0);
                info.setPersonInChargeType(PersonInchargeUtils.setSuperior2PersonInChargeType(info.getPersonInChargeType()));
                info.setPersonInchargePosition((String) null);
                info.setPointPosition((String) null);
                info.put("personIncharge", new PerfEvaPersonInchargeCollection());
                croeBase.add(info);
            }

            this.runBatchSaveEvaObject(ctx, periodId, hrOrgUnit, croeBase);
        } else {
            throw new SHRWebException((new SHRPerfWebBizException(SHRPerfWebBizException.GETERR)).getMessage());
        }
    }

    protected void runBatchSaveEvaObject(Context ctx, String periodId, String hrOrgUnit, CoreBaseCollection croeBase) throws SHRWebException {
        logger.error("runBatchSaveEvaObject----");
        int sucess = 0;
        int failure = 0;
        String billId = "";
        try {
            Set<String> sucessIds = new HashSet();
            Map<String, String> verifyMap = new HashMap();
            Map<String, String> evaPlanNameMap = this.getPerfPlanName(ctx, periodId, hrOrgUnit);
            CoreBaseCollection addCoreColl = new CoreBaseCollection();
            int count = croeBase.size();
            int j = 1;
            Set<String> personIds = new HashSet();

            int i;
            PerfEvaObjectInfo info;
            for (i = 0; i < count; ++i) {
                info = (PerfEvaObjectInfo) croeBase.get(i);
                personIds.add(info.getPerson().getId().toString());
                if (j != 500 && i != count - 1) {
                    ++j;
                } else {
                    Map<String, String> tempMap = this.getEvaObject(ctx, periodId, hrOrgUnit, personIds);
                    if (!CollectionUtils.isEmpty(tempMap)) {
                        verifyMap.putAll(tempMap);
                    }

                    j = 1;
                    personIds = new HashSet();
                }
            }
            IObjectPK[] iObjectPKS = null;
            for (i = 0; i < croeBase.size(); ++i) {
                info = (PerfEvaObjectInfo) croeBase.get(i);
                BatchMessageTipsBody body = new BatchMessageTipsBody();

                try {
                    String id = info.getPerfFile().getId().toString();
                    billId = billId + id + ",";
                    body.setId(id);
                    if (sucessIds.contains(id)) {
                        continue;
                    }

                    sucessIds.add(id);

                    String key = info.getPerson().getNumber() + "_" + info.getPerson().getName() + "_" + info.getPerfFile().getPosition().getId().toString() + "_" + info.getPerfFile().getAssignType().getValue();
                    if (!org.apache.commons.lang.StringUtils.isEmpty((String) verifyMap.get(key))) {
                        continue;
                        //throw new BOSException((new SHRPerfWebBizException(SHRPerfWebBizException.EVAOBJALREADYADDED, new Object[]{evaPlanNameMap.get(verifyMap.get(key))})).getMessage());
                    }

                    addCoreColl.add(info);
                    ++sucess;
                    body.setMuitTipsState(Boolean.TRUE);
                    body.setMuitTipsMessage(SHRPerfWebResourceUtil.getRes(SHRPerfWebResource.OPERATESUCCESS, ctx));
                } catch (Exception var23) {
                    var23.printStackTrace();
                    croeBase.removeObject(i);
                    --i;
                    ++failure;
                    body.setMuitTipsState(Boolean.FALSE);
                    if (var23 instanceof ObjectNotFoundException) {
                        body.setMuitTipsMessage(SHRPerfWebResourceUtil.getRes(SHRPerfWebResource.DATADELETED, ctx));
                    } else {
                        body.setMuitTipsMessage(SHRPerfWebResourceUtil.getRes(SHRPerfWebResource.OPERATEFAILED, ctx));
                    }

                    logger.error(var23.getMessage(), var23);
                }

                if (addCoreColl.size() == 100) {
                    iObjectPKS = PerfEvaObjectFactory.getLocalInstance(ctx).saveBatchData(addCoreColl);
                    addCoreColl.clear();
                }
            }

            if (addCoreColl.size() > 0) {
                iObjectPKS = PerfEvaObjectFactory.getLocalInstance(ctx).saveBatchData(addCoreColl);
                addCoreColl.clear();
            }

            if (sucess > 0) {
                PerfPlanUtils.writeRefStateForPerfPeriod(ctx, periodId, RefStatusEnum.REFERED);
            }
            if (iObjectPKS != null) {
                setUpGroups(ctx, iObjectPKS);
            }
        } catch (Exception var24) {
            logger.error("runBatchSaveEvaObject----" + var24);
            var24.printStackTrace();
            throw new SHRWebException(var24.getMessage(), var24);
        }
        logger.error("runBatchSaveEvaObject----end");

    }

    /**
     * ÷
     *
     * @param ctx
     * @param iObjectPKS
     */
    public void setUpGroups(Context ctx, IObjectPK[] iObjectPKS) throws BOSException, EASBizException, ShrWebBizException {
        IPerfGoalsGroup iPerfGoalsGroup = PerfGoalsGroupFactory.getLocalInstance(ctx);
        PerfGoalsGroupInfo perfGoalsGroupInfo = iPerfGoalsGroup.getPerfGoalsGroupCollection("where name = '˾ȼЧĿ' order by createTime desc").get(0);
        String goalsGroupId = perfGoalsGroupInfo.getId().toString();
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < iObjectPKS.length; i++) {
            IObjectPK iObjectPK = iObjectPKS[i];
            sb.append(iObjectPK.toString()).append(",");
        }
        sb.deleteCharAt(sb.lastIndexOf(","));
        String groupId = goalsGroupId;
        if (!PerfEvaObjGroupUtils.checkGroupIsValid(ctx, groupId)) {
            throw new ShrWebBizException((new SHRPerfWebBizException(SHRPerfWebBizException.INDECATORNOTSAVEDINSELECTGROUP)).getMessage());
        } else {
            try {
                CoreBaseCollection perfEvaObjColl = new CoreBaseCollection();
                IPerfEvaObject iPerfEvaObject = PerfEvaObjectFactory.getLocalInstance(ctx);
                Set<String> billIds = new HashSet();
                Set<String> fails = new HashSet();
                String billId = sb.toString();
                billIds.addAll(Arrays.asList(billId.split(",")));
                PerfEvaObjectCollection coll = getPerfEvaObjectCollectionByIds(ctx, billIds);
                if (coll != null && coll.size() > 0) {
                    String periodId = coll.get(0).getPeriod().getId().toString();
                    PerfPeriodInfo perfPeriodInfo = PerfPeriodFactory.getLocalInstance(ctx).getPerfPeriodInfo("select perfPlan.id,perfRule.id where id = '" + periodId + "'");
                    String perfRuleId = perfPeriodInfo.getPerfRule().getId().toString();
                    Map<String, Object> checkResult = checkRuleData(perfRuleId, "", perfPeriodInfo.getPerfPlan().getId().toString(), ctx);
                    boolean flag = (Boolean) checkResult.get("result");
                    String reMsg = "";
                    if (!flag) {
                        //addBatchMessageTips(billIds, (String) checkResult.get("msg"), batchMessageTipsHeader);
                    } else {
                        int i = 0;
                        while (true) {
                            if (i >= coll.size()) {
                                iPerfEvaObject.updateBatchData(perfEvaObjColl);
                                break;
                            }
                            PerfEvaObjectInfo evaObjInfo = coll.get(i);
                            PerfEvaObjectUnitGoalsStatusEnum unitGoalsStatus = evaObjInfo.getUnitGoalsStatus();
                            if (unitGoalsStatus != null && unitGoalsStatus.getValue() == 1) {
                                fails.add(evaObjInfo.getId().toString());
                            } else {
                                evaObjInfo.setBelongGroup(groupId);
                                perfEvaObjColl.add(evaObjInfo);
                            }
                            ++i;
                        }
                    }
                }
            } catch (BOSException var22) {
                logger.error("", var22);
                throw new ShrWebBizException((new SHRPerfWebBizException(SHRPerfWebBizException.SYSTEMERR)).getMessage());
            } catch (EASBizException var23) {
                logger.error("", var23);
                throw new ShrWebBizException((new SHRPerfWebBizException(SHRPerfWebBizException.SYSTEMERR)).getMessage());
            }
        }
    }

    private static PerfEvaObjectCollection getPerfEvaObjectCollectionByIds(Context ctx, Set<String> billIds) throws BOSException {
        FilterInfo filterInfo = new FilterInfo();
        filterInfo.getFilterItems().add(new FilterItemInfo("id", billIds, CompareType.INCLUDE));
        EntityViewInfo evi = new EntityViewInfo();
        evi.setFilter(filterInfo);
        IPerfEvaObject iPerfEvaObject = PerfEvaObjectFactory.getLocalInstance(ctx);
        return iPerfEvaObject.getPerfEvaObjectCollection(evi);
    }

    public static void checkPeriod(Context ctx, String period) throws SHRWebException {
        if (org.apache.commons.lang.StringUtils.isEmpty(period)) {
            throw new ShrWebBizException((new SHRPerfWebBizException(SHRPerfWebBizException.PERFPERIODNOTEXIST)).getMessage());
        } else {
            PerfPeriodInfo info = null;

            try {
                info = PerfPeriodFactory.getLocalInstance(ctx).getPerfPeriodInfo("where id = '" + period + "'");
            } catch (Exception var3) {
                logger.error(var3.toString());
                throw new ShrWebBizException((new SHRPerfWebBizException(SHRPerfWebBizException.GETPERIODDATAERR)).getMessage());
            }

            if (info.getEvaPeriodStatus().getValue() == 100) {
                throw new ShrWebBizException((new SHRPerfWebBizException(SHRPerfWebBizException.OPERATENOTALLOWEDASPERIODISEND)).getMessage());
            }
        }
    }

    public Map<String, String> getPerfPlanName(Context ctx, String period, String hrOrgUnit) throws EASBizException, BOSException {
        String oql = this.getExistPerfplanSql(ctx, period, hrOrgUnit);
        Map<String, String> map = new HashMap();
        EntityViewInfo viewInfo = new EntityViewInfo();
        SelectorItemCollection sic = new SelectorItemCollection();
        sic.add(new SelectorItemInfo("id"));
        sic.add(new SelectorItemInfo("name"));
        FilterInfo filter = new FilterInfo();
        filter.getFilterItems().add(new FilterItemInfo("id", oql, CompareType.INNER));
        viewInfo.setFilter(filter);
        viewInfo.setSelector(sic);
        PerfPlanCollection coll = PerfPlanFactory.getLocalInstance(ctx).getPerfPlanCollection(viewInfo);

        for (int i = 0; i < coll.size(); ++i) {
            map.put(coll.get(i).getId().toString(), coll.get(i).getName());
        }

        return map;
    }

    private String getExistPerfplanSql(Context ctx, String periodId, String hrOrgUnit) throws BOSException, EASBizException {
        PerfPeriodInfo perfPeriodInfo = null;
        PerfPlanInfo planInfo = null;
        perfPeriodInfo = PerfPeriodFactory.getLocalInstance(ctx).getPerfPeriodInfo(new ObjectUuidPK(periodId));
        planInfo = PerfPlanFactory.getLocalInstance(ctx).getPerfPlanInfo(new ObjectUuidPK(perfPeriodInfo.getPerfPlan().getId().toString()));
        String sql = " select tt.fid from t_eva_perfperiod t   inner join t_eva_perfplan tt     on t.fperfplan = tt.fid   where t.fnumber ='" + perfPeriodInfo.getNumber() + "'     and tt.fperfmodelid = '" + planInfo.getPerfModel().getId().toString() + "'    and tt.FHrUnitOrgID = '" + hrOrgUnit + "'";
        return sql;
    }

    protected Map<String, String> getEvaObject(Context ctx, String period, String hrOrgUnit, Set<String> personIds) throws EASBizException, BOSException {
        String oql = this.getExistPeriodSql(ctx, period, hrOrgUnit);
        Map<String, String> map = new HashMap();
        EntityViewInfo viewInfo = new EntityViewInfo();
        SelectorItemCollection sic = new SelectorItemCollection();
        sic.add(new SelectorItemInfo("id"));
        sic.add(new SelectorItemInfo("person.name"));
        sic.add(new SelectorItemInfo("person.number"));
        sic.add(new SelectorItemInfo("perfFile.position.id"));
        sic.add(new SelectorItemInfo("perfFile.assignType"));
        sic.add(new SelectorItemInfo("period.perfplan.id"));
        FilterInfo filter = new FilterInfo();
        filter.getFilterItems().add(new FilterItemInfo("period", oql, CompareType.INNER));
        filter.getFilterItems().add(new FilterItemInfo("isOrg", 1, CompareType.NOTEQUALS));
        filter.getFilterItems().add(new FilterItemInfo("person.id", personIds, CompareType.INCLUDE));
        viewInfo.setFilter(filter);
        viewInfo.setSelector(sic);
        PerfEvaObjectCollection coll = PerfEvaObjectFactory.getLocalInstance(ctx).getPerfEvaObjectCollection(viewInfo);

        for (int i = 0; i < coll.size(); ++i) {
            PerfEvaObjectInfo info = coll.get(i);
            if (!org.apache.commons.lang.StringUtils.isEmpty(info.getPerson().getNumber()) && info.getPerfFile().getPosition() != null && info.getPerfFile().getAssignType() != null) {
                String key = info.getPerson().getNumber() + "_" + info.getPerson().getName() + "_" + info.getPerfFile().getPosition().getId().toString() + "_" + info.getPerfFile().getAssignType().getValue();
                map.put(key, info.getPeriod().getPerfPlan().getId().toString());
            }
        }

        return map;
    }

    private String getExistPeriodSql(Context ctx, String periodId, String hrOrgUnit) throws BOSException, EASBizException {
        PerfPeriodInfo perfPeriodInfo = null;
        PerfPlanInfo planInfo = null;
        perfPeriodInfo = PerfPeriodFactory.getLocalInstance(ctx).getPerfPeriodInfo(new ObjectUuidPK(periodId));
        planInfo = PerfPlanFactory.getLocalInstance(ctx).getPerfPlanInfo(new ObjectUuidPK(perfPeriodInfo.getPerfPlan().getId().toString()));
        String sql = " select t.fid from t_eva_perfperiod t    left outer join t_eva_perfplan tt     on t.fperfplan = tt.fid   where t.fnumber ='" + perfPeriodInfo.getNumber() + "'     and tt.fperfmodelid = '" + planInfo.getPerfModel().getId().toString() + "'    and tt.FHrUnitOrgID = '" + hrOrgUnit + "'";
        return sql;
    }

    public static Map<String, Object> checkRuleData(String ruleId, String excludeAreaBaseId, String perfPlanId, Context ctx) {
        Map<String, Object> result = new HashMap<>();
        if (StringUtils.isEmpty(ruleId)) {
            result.put("result", Boolean.valueOf(false));
            result.put("msg", SHRPerfWebResourceUtil.getRes(SHRPerfWebResource.PRSUENABLEFAILEDMESSAGE5, ctx));
            return result;
        }
        Double sum = new Double(0.0D);
        String type = "NOWEIGHTCONTROL";
        EvaluFormAreaBaseCollection coll = null;
        EvaluFormAreaBaseCollection coll1 = null;
        EvaluFormAreaBaseCollection allColl = null;
        result.put("result", Boolean.valueOf(false));
        result.put("msg", "");
        int unvalidWeightCount = 0;
        boolean weightForce100 = true;
        try {
            IEvaluFormAreaBase iEvaluFormAreaBase = EvaluFormAreaBaseFactory.getLocalInstance(ctx);
            if (!StringUtils.isEmpty(perfPlanId)) {
                PerfPlanCollection perfPlanCollection = PerfPlanFactory.getLocalInstance(ctx).getPerfPlanCollection(" where id ='" + perfPlanId + "'");
                if (perfPlanCollection.size() > 0) {
                    PerfPlanInfo perfPlanInfo = perfPlanCollection.get(0);
                    weightForce100 = perfPlanInfo.isWeightForce100();
                }
            }
            String otherParam = "";
            if (!StringUtils.isEmpty(excludeAreaBaseId))
                otherParam = " and id <> '" + excludeAreaBaseId + "'";
            allColl = iEvaluFormAreaBase.getEvaluFormAreaBaseCollection("where performanceRule='" + ruleId + "'" + otherParam);
            coll1 = iEvaluFormAreaBase.getEvaluFormAreaBaseCollection("where performanceRule='" + ruleId + "' " + otherParam + "  and (sumScoreRule=300 or sumScoreRule=600 or canRate=1)");
            coll = iEvaluFormAreaBase.getEvaluFormAreaBaseCollection("where performanceRule='" + ruleId + "' " + otherParam + " and weightControl=1 and (sumScoreRule=300 or sumScoreRule=600)");
            if (allColl.size() > 1 && allColl.size() - coll.size() == 1)
                type = "ISWEIGHTCONTROL";
            EvaluFormAreaBaseInfo evaluFormAreaBaseInfo = null;
            int size = allColl.size();
            for (int i = 0; i < size; i++) {
                evaluFormAreaBaseInfo = allColl.get(i);
                if (evaluFormAreaBaseInfo.getAreaType().getValue().equals("100") || evaluFormAreaBaseInfo.getAreaType().getValue().equals("400") || evaluFormAreaBaseInfo.getSumScoreRule().getValue().equals("400") || evaluFormAreaBaseInfo
                        .getSumScoreRule().getValue().equals("500") || YesOrNoEnum.NO.equals(evaluFormAreaBaseInfo.getCanRate())) {
                    unvalidWeightCount++;
                } else {
                    Double _sum = Double.valueOf(StringUtils.isEmpty(evaluFormAreaBaseInfo.getAreaWeight()) ? "0" : evaluFormAreaBaseInfo.getAreaWeight());
                    sum = Double.valueOf(sum.doubleValue() + _sum.doubleValue());
                }
            }
        } catch (BOSException e) {
            logger.error("BOSException: ", (Throwable) e);
            result.put("error", SHRPerfWebResourceUtil.getRes(SHRPerfWebResource.AREAVERIFYERR, ctx));
            return result;
        }
        if (allColl.size() == 1) {
            result.put("msg", SHRPerfWebResourceUtil.getRes(SHRPerfWebResource.PRSUENABLEFAILEDMESSAGE1, ctx));
            return result;
        }
        if (allColl.size() == 2 && coll1.size() == 0) {
            result.put("msg", SHRPerfWebResourceUtil.getRes(SHRPerfWebResource.PRSUENABLEFAILEDMESSAGE2, ctx));
            return result;
        }
        PerformanceRuleInfo ruleInfo = GetEntityInfoUtils.getPerformanceRuleInfo(ctx, ruleId);
        int scoreWay = ruleInfo.getScoringWay().getValue();
        if (type.equals("ISWEIGHTCONTROL")) {
            if (scoreWay == 1) {
                BigDecimal tmpSumScore = new BigDecimal(sum.doubleValue());
                PerfScoreInfo scoreInfo = GetEntityInfoUtils.getPerfScoreInfoById(ctx, ruleInfo.getPariMutual().getId().toString());
                if (coll.size() > 0 && tmpSumScore.compareTo(scoreInfo.getMax()) < 0) {
                    result.put("msg", SHRPerfWebResourceUtil.getRes(SHRPerfWebResource.PRSUENABLEFAILEDMESSAGE7, ctx));
                    return result;
                }
                if (tmpSumScore.compareTo(scoreInfo.getMax()) > 0) {
                    result.put("msg", SHRPerfWebResourceUtil.getRes(SHRPerfWebResource.PRSUENABLEFAILEDMESSAGE8, ctx));
                    return result;
                }
            } else {
                if (coll.size() > 0 && sum.doubleValue() < 100.0D) {
                    result.put("msg", SHRPerfWebResourceUtil.getRes(SHRPerfWebResource.PRSUENABLEFAILEDMESSAGE3, ctx));
                    return result;
                }
                if (sum.doubleValue() > 100.0D) {
                    result.put("msg", SHRPerfWebResourceUtil.getRes(SHRPerfWebResource.PRSUENABLEFAILEDMESSAGE4, ctx));
                    return result;
                }
            }
        } else if (scoreWay == 1) {
            if (weightForce100) {
                BigDecimal tmpSumScore = new BigDecimal(sum.doubleValue());
                PerfScoreInfo scoreInfo = GetEntityInfoUtils.getPerfScoreInfoById(ctx, ruleInfo.getPariMutual().getId().toString());
                if (tmpSumScore.compareTo(scoreInfo.getMax()) > 0) {
                    result.put("msg", SHRPerfWebResourceUtil.getRes(SHRPerfWebResource.PRSUENABLEFAILEDMESSAGE7, ctx));
                    return result;
                }
                if (tmpSumScore.compareTo(scoreInfo.getMax()) < 0 && allColl.size() - coll.size() == unvalidWeightCount) {
                    result.put("msg", SHRPerfWebResourceUtil.getRes(SHRPerfWebResource.PRSUENABLEFAILEDMESSAGE9, ctx));
                    return result;
                }
            }
        } else if (weightForce100) {
            if (sum.doubleValue() > 100.0D) {
                result.put("msg", SHRPerfWebResourceUtil.getRes(SHRPerfWebResource.PRSUENABLEFAILEDMESSAGE3, ctx));
                return result;
            }
            if (sum.doubleValue() < 100.0D && allColl.size() - coll.size() == unvalidWeightCount) {
                result.put("msg", SHRPerfWebResourceUtil.getRes(SHRPerfWebResource.PRSUENABLEFAILEDMESSAGE6, ctx));
                return result;
            }
        }
        result.put("result", Boolean.valueOf(true));
        result.put("message", "");
        return result;
    }
}
